/*

UNIT HD44790

Basic I/O routines for HD44780 industrial lcd-display

Author:	Patrick Lindenberg
Date:	06/16/07

Copyright 2007 by Lindenberg Electronics
All Rights Reserved


*/

#include "main.h"
#include <avr/io.h>
#include <util/delay.h>
#include "hd44780.h"

void lcd_data(unsigned char data)
{
   unsigned char tmp = data;

   LCD_PORT = 0x00;  // reset port	
 
   data = data >> 4;
   data = data & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= data | (1<<LCD_RS); // inkl. LCD_RS = RS-Leitung setzen
   lcd_enable();
 
   tmp = tmp & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= tmp | (1<<LCD_RS); // inkl. LCD_RS = RS-Leitung setzen
   lcd_enable();
   
   LCD_PORT = 0x00;	   

   _delay_us(50);
}

void lcd_command(unsigned char cmd)
{
   unsigned char tmp = cmd;
 
   LCD_PORT = 0x00;  // reset port	

   cmd = cmd >> 4;   // oberes Nibble holen
   cmd = cmd & 0x0F; // maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= cmd;  // setzen
   lcd_enable();
 
   tmp = tmp & 0x0F; // unteres Nibble holen und maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= tmp;   // setzen
   lcd_enable();
   
   LCD_PORT = 0x00;

   _delay_us(50);
}

void lcd_enable(void)
{
   	LCD_PORT |= (1<<5);
	asm("nop");
	asm("nop");
	asm("nop");
   	LCD_PORT &= ~(1<<5);
}


void lcd_init(void)
{
	unsigned char i;

	LCD_PORT_IO = 0xff; // lcd_port auf output setzen

	for(i=0;i<250;i++) _delay_ms(1);

	LCD_PORT &= 0xF0;	
	LCD_PORT |= 0x03;
	lcd_enable();
	LCD_PORT |= 0x03;
	lcd_enable();
	LCD_PORT |= 0x03;
	lcd_enable();


	
	// 4bit-Modus einstellen
   	LCD_PORT |= 0x02;       // 0b00000010  
   	lcd_enable();


	_delay_ms(5);
	lcd_command(0x28);
	_delay_ms(5);
	lcd_command(0x0C);
	_delay_ms(5);
	lcd_command(0b00001100);
}


void lcd_clear(void)
{
	lcd_command(CMD_CLEAR);
	_delay_ms(5);
}


void lcd_home(void)
{
	lcd_command(CMD_HOME);
	_delay_ms(5);
}


void lcd_setline(unsigned char line)
{
	switch(line) 
		{
		case 1 : { lcd_command(0x00); break; }
		case 2 : { lcd_command(0xC0); break; }
//		case 3 : { lcd_command(0x94); break; }
//		case 4 : { lcd_command(0xD4); break; }

		/* to be expandable */
		}
}


void lcd_putstr(char *s)
{
 	while (*s)
    {   
	    lcd_data(*s);
        s++;
    }

}

